@@ -13,9 +13,7 @@
  * You should have received a copy of the GNU General Public License along with
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#ifdef HAVE_CONFIG_H
-#  include "libfswatch_config.h"
-#endif
+#include "libfswatch/libfswatch_config.h"
 #include "gettext.h"
 #include "fswatch.hpp"
 #include <iostream>
@@ -27,6 +25,7 @@
 #include <ctime>
 #include <cerrno>
 #include <vector>
+#include <array>
 #include <map>
 #include "libfswatch/c++/path_utils.hpp"
 #include "libfswatch/c++/event.hpp"
@@ -36,6 +35,9 @@
 #include "libfswatch/c/libfswatch.h"
 #include "libfswatch/c/libfswatch_log.h"
 #include "libfswatch/c++/libfswatch_exception.hpp"
+#ifdef HAVE_FSEVENTS_FSEVENTSTREAMSETDISPATCHQUEUE
+  #include "libfswatch/c++/fsevents_monitor.hpp"
+#endif
 
 #ifdef HAVE_GETOPT_LONG
 #  include <getopt.h>
@@ -85,6 +87,7 @@ static bool _1flag = false;
 static bool aflag = false;
 static bool allow_overflow = false;
 static int batch_marker_flag = false;
+static bool bflag = false;
 static bool dflag = false;
 static bool Eflag = false;
 static bool fieFlag = false;
@@ -92,6 +95,7 @@ static bool Iflag = false;
 static bool Lflag = false;
 static bool mflag = false;
 static bool nflag = false;
+static bool noDeferFlag = false;
 static bool oflag = false;
 static bool rflag = false;
 static bool tflag = false;
@@ -120,6 +124,7 @@ static const int OPT_ALLOW_OVERFLOW = 132;
 static const int OPT_MONITOR_PROPERTY = 133;
 static const int OPT_FIRE_IDLE_EVENTS = 134;
 static const int OPT_FILTER_FROM = 135;
+static const int OPT_NO_DEFER = 136;
 
 static void list_monitor_types(std::ostream& stream)
 {
@@ -132,7 +137,7 @@ static void list_monitor_types(std::ostream& stream)
 static void print_version(std::ostream& stream)
 {
   stream << PACKAGE_STRING << "\n";
-  stream << "Copyright (C) 2013-2018 Enrico M. Crisostomo <enrico.m.crisostomo@gmail.com>.\n";
+  stream << "Copyright (C) 2013-2021 Enrico M. Crisostomo <enrico.m.crisostomo@gmail.com>.\n";
   stream << _("License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n");
   stream << _("This is free software: you are free to change and redistribute it.\n");
   stream << _("There is NO WARRANTY, to the extent permitted by law.\n");
@@ -154,6 +159,7 @@ static void usage(std::ostream& stream)
   stream << "     --allow-overflow  " << _("Allow a monitor to overflow and report it as a change event.\n");
   stream << "     --batch-marker    " << _("Print a marker at the end of every batch.\n");
   stream << " -a, --access          " << _("Watch file accesses.\n");
+  stream << " -b, --bubble-events   " << _("Bubble events with the same timestamp and path.\n");
   stream << " -d, --directories     " << _("Watch directories only.\n");
   stream << " -e, --exclude=REGEX   " << _("Exclude paths matching REGEX.\n");
   stream << " -E, --extended        " << _("Use extended regular expressions.\n");
@@ -166,6 +172,9 @@ static void usage(std::ostream& stream)
   stream << " -i, --include=REGEX   " << _("Include paths matching REGEX.\n");
   stream << " -I, --insensitive     " << _("Use case insensitive regular expressions.\n");
   stream << " -l, --latency=DOUBLE  " << _("Set the latency.\n");
+  #if defined(HAVE_FSEVENTS_FSEVENTSTREAMSETDISPATCHQUEUE)
+  stream << "     --no-defer        " << _("Set the no defer flag in the monitor.\n");
+  #endif
   stream << " -L, --follow-links    " << _("Follow symbolic links.\n");
   stream << " -M, --list-monitors   " << _("List the available monitors.\n");
   stream << " -m, --monitor=NAME    " << _("Use the specified monitor.\n");
@@ -184,7 +193,7 @@ static void usage(std::ostream& stream)
   stream << "     --version         " << _("Print the version of ") << PACKAGE_NAME << _(" and exit.\n");
   stream << "\n";
 #else
-  std::string option_string = "[01adeEfhilLMmnortuvx]";
+  std::string option_string = "[01abdeEfhilLMmnortuvx]";
 
   stream << PACKAGE_STRING << "\n\n";
   stream << _("Usage:\n");
@@ -194,6 +203,7 @@ static void usage(std::ostream& stream)
   stream << " -0  Use the ASCII NUL character (0) as line separator.\n";
   stream << " -1  Exit fswatch after the first set of events is received.\n";
   stream << " -a  Watch file accesses.\n";
+  stream << " -b  Bubble events with the same timestamp and path.\n";
   stream << " -d  Watch directories only.\n";
   stream << " -e  Exclude paths matching REGEX.\n";
   stream << " -E  Use extended regular expressions.\n";
@@ -336,15 +346,16 @@ static void print_event_timestamp(const event& evt)
 {
   const time_t& evt_time = evt.get_time();
 
-  char time_format_buffer[TIME_FORMAT_BUFF_SIZE];
+  std::array<char, TIME_FORMAT_BUFF_SIZE> time_format_buffer{};
   const struct tm *tm_time = uflag ? gmtime(&evt_time) : localtime(&evt_time);
 
   std::string date =
-    strftime(time_format_buffer,
-             TIME_FORMAT_BUFF_SIZE,
+    strftime(time_format_buffer.data(),
+             time_format_buffer.size(),
              tformat.c_str(),
-             tm_time) ? std::string(time_format_buffer) : std::string(
-      _("<date format error>"));
+             tm_time)
+             ? std::string(time_format_buffer.data())
+             : std::string(_("<date format error>"));
 
   std::cout << date;
 }
@@ -421,7 +432,7 @@ static void write_events(const std::vector<event>& events)
   }
 }
 
-static void process_events(const std::vector<event>& events, void *context)
+static void process_events(const std::vector<event>& events, void *)
 {
   if (oflag)
     write_one_batch_event(events);
@@ -483,6 +494,10 @@ static void start_monitor(int argc, char **argv, int optind)
   active_monitor->set_properties(monitor_properties);
   active_monitor->set_allow_overflow(allow_overflow);
   active_monitor->set_latency(lvalue);
+  #if defined(HAVE_FSEVENTS_FSEVENTSTREAMSETDISPATCHQUEUE)
+  if (noDeferFlag)
+    active_monitor->set_property(std::string(fsw::fsevents_monitor::DARWIN_EVENTSTREAM_NO_DEFER), "true");
+  #endif
   active_monitor->set_fire_idle_event(fieFlag);
   active_monitor->set_recursive(rflag);
   active_monitor->set_directory_only(dflag);
@@ -490,6 +505,7 @@ static void start_monitor(int argc, char **argv, int optind)
   active_monitor->set_filters(filters);
   active_monitor->set_follow_symlinks(Lflag);
   active_monitor->set_watch_access(aflag);
+  active_monitor->set_bubble_events(bflag);
 
   active_monitor->start();
 }
@@ -497,7 +513,7 @@ static void start_monitor(int argc, char **argv, int optind)
 static void parse_opts(int argc, char **argv)
 {
   int ch;
-  std::string short_options = "01ade:Ef:hi:Il:LMm:nortuvx";
+  std::string short_options = "01abde:Ef:hi:Il:LMm:nortuvx";
 
 #ifdef HAVE_GETOPT_LONG
   int option_index = 0;
@@ -505,6 +521,7 @@ static void parse_opts(int argc, char **argv)
     {"access",               no_argument,       nullptr,       'a'},
     {"allow-overflow",       no_argument,       nullptr,       OPT_ALLOW_OVERFLOW},
     {"batch-marker",         optional_argument, nullptr,       OPT_BATCH_MARKER},
+    {"bubble-events",        no_argument,       nullptr,       'b'},
     {"directories",          no_argument,       nullptr,       'd'},
     {"event",                required_argument, nullptr,       OPT_EVENT_TYPE},
     {"event-flags",          no_argument,       nullptr,       'x'},
@@ -520,6 +537,9 @@ static void parse_opts(int argc, char **argv)
     {"include",              required_argument, nullptr,       'i'},
     {"insensitive",          no_argument,       nullptr,       'I'},
     {"latency",              required_argument, nullptr,       'l'},
+  #ifdef HAVE_FSEVENTS_FSEVENTSTREAMSETDISPATCHQUEUE
+    {"no-defer",             no_argument,       nullptr,       OPT_NO_DEFER},
+  #endif
     {"list-monitors",        no_argument,       nullptr,       'M'},
     {"monitor",              required_argument, nullptr,       'm'},
     {"monitor-property",     required_argument, nullptr,       OPT_MONITOR_PROPERTY},
@@ -560,6 +580,10 @@ static void parse_opts(int argc, char **argv)
       aflag = true;
       break;
 
+    case 'b':
+      bflag = true;
+      break;
+
     case 'd':
       dflag = true;
       break;
@@ -677,6 +701,10 @@ static void parse_opts(int argc, char **argv)
       filter_files.emplace_back(optarg);
       break;
 
+    case OPT_NO_DEFER:
+      noDeferFlag = true;
+      break;
+
     case '?':
       usage(std::cerr);
       exit(FSW_EXIT_UNK_OPT);
