@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License along with
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#include "gettext_defs.h"
+#include "libfswatch/gettext_defs.h"
 #include "event.hpp"
 #include "libfswatch_exception.hpp"
 #include <map>
@@ -22,15 +22,12 @@ using namespace std;
 
 namespace fsw
 {
-
   event::event(string path, time_t evt_time, vector<fsw_event_flag> flags) :
-    path(path), evt_time(evt_time), evt_flags(flags)
+    path(std::move(path)), evt_time(evt_time), evt_flags(std::move(flags))
   {
   }
 
-  event::~event()
-  {
-  }
+  event::~event() = default;
 
   string event::get_path() const
   {
@@ -47,7 +44,7 @@ namespace fsw
     return evt_flags;
   }
 
-  fsw_event_flag event::get_event_flag_by_name(const string &name)
+  fsw_event_flag event::get_event_flag_by_name(const string& name)
   {
 #define FSW_MAKE_PAIR_FROM_NAME(p) {#p, p}
     static const map<string, fsw_event_flag> flag_by_names = {
@@ -77,7 +74,7 @@ namespace fsw
     return flag->second;
   }
 
-  string event::get_event_flag_name(const fsw_event_flag &flag)
+  string event::get_event_flag_name(const fsw_event_flag& flag)
   {
 #define FSW_MAKE_PAIR_FROM_NAME(p) {p, #p}
     static const map<fsw_event_flag, string> names_by_flag = {
