@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015 Enrico M. Crisostomo
+ * Copyright (c) 2015-2021 Enrico M. Crisostomo
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -13,9 +13,7 @@
  * You should have received a copy of the GNU General Public License along with
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#ifdef HAVE_CONFIG_H
-#  include "libfswatch_config.h"
-#endif
+#include "libfswatch/libfswatch_config.h"
 
 #ifdef HAVE_PORT_H
 
@@ -28,12 +26,12 @@
 #  include <port.h>
 #  include <sys/types.h>
 #  include <sys/stat.h>
-#  include "gettext_defs.h"
+#  include "libfswatch/gettext_defs.h"
 #  include "fen_monitor.hpp"
 #  include "libfswatch_map.hpp"
 #  include "libfswatch_set.hpp"
 #  include "libfswatch_exception.hpp"
-#  include "../c/libfswatch_log.h"
+#  include "libfswatch/c/libfswatch_log.h"
 #  include "path_utils.hpp"
 
 using namespace std;
@@ -125,8 +123,6 @@ namespace fsw
 
   static const vector<FenFlagType> event_flag_type = create_flag_type_vector();
 
-  REGISTER_MONITOR_IMPL(fen_monitor, fen_monitor_type);
-
   fen_monitor::fen_monitor(vector<string> paths_to_monitor,
                            FSW_EVENT_CALLBACK * callback,
                            void * context) :
@@ -401,7 +397,7 @@ namespace fsw
           throw libfsw_exception(msg);
         }
       }
-      else if (errno != ETIME) perror("port_get");
+      else if (errno != ETIME && errno != EINTR) perror("port_get");
     }
   }
 }
