@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2021 Enrico M. Crisostomo
+ * Copyright (c) 2014-2022 Enrico M. Crisostomo
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -13,15 +13,15 @@
  * You should have received a copy of the GNU General Public License along with
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#ifdef HAVE_CONFIG_H
-#  include "libfswatch_config.h"
-#endif
-
+#include "libfswatch/libfswatch_config.h"
 #include <memory>
+#include <unistd.h> // isatty()
+#include <cstdio> // fileno()
+#include <thread>
 #include "fsevents_monitor.hpp"
-#include "gettext_defs.h"
+#include "libfswatch/gettext_defs.h"
 #include "libfswatch_exception.hpp"
-#include "c/libfswatch_log.h"
+#include "libfswatch/c/libfswatch_log.h"
 #  ifdef HAVE_CXX_MUTEX
 #    include <mutex>
 #  endif
@@ -41,7 +41,7 @@ namespace fsw
   static vector<FSEventFlagType> create_flag_type_vector()
   {
     vector<FSEventFlagType> flags;
-#ifdef MACOS_GE_10_5
+#ifdef HAVE_MACOS_GE_10_5
     flags.push_back({kFSEventStreamEventFlagNone, fsw_event_flag::PlatformSpecific});
     flags.push_back({kFSEventStreamEventFlagMustScanSubDirs, fsw_event_flag::PlatformSpecific});
     flags.push_back({kFSEventStreamEventFlagUserDropped, fsw_event_flag::PlatformSpecific});
@@ -53,7 +53,7 @@ namespace fsw
     flags.push_back({kFSEventStreamEventFlagUnmount, fsw_event_flag::PlatformSpecific});
 #endif
 
-#ifdef MACOS_GE_10_7
+#ifdef HAVE_MACOS_GE_10_7
     flags.push_back({kFSEventStreamEventFlagItemChangeOwner, fsw_event_flag::OwnerModified});
     flags.push_back({kFSEventStreamEventFlagItemCreated, fsw_event_flag::Created});
     flags.push_back({kFSEventStreamEventFlagItemFinderInfoMod, fsw_event_flag::PlatformSpecific});
@@ -68,11 +68,11 @@ namespace fsw
     flags.push_back({kFSEventStreamEventFlagItemXattrMod, fsw_event_flag::AttributeModified});
 #endif
 
-#ifdef MACOS_GE_10_9
+#ifdef HAVE_MACOS_GE_10_9
     flags.push_back({kFSEventStreamEventFlagOwnEvent, fsw_event_flag::AttributeModified});
 #endif
 
-#ifdef MACOS_GE_10_10
+#ifdef HAVE_MACOS_GE_10_10
     flags.push_back({kFSEventStreamEventFlagItemIsHardlink, fsw_event_flag::Link});
     flags.push_back({kFSEventStreamEventFlagItemIsLastHardlink, fsw_event_flag::Link});
     flags.push_back({kFSEventStreamEventFlagItemIsLastHardlink, fsw_event_flag::PlatformSpecific});
@@ -121,14 +121,9 @@ namespace fsw
     if (!stream)
       throw libfsw_exception(_("Event stream could not be created."));
 
-    // Fire the event loop
-    run_loop = CFRunLoopGetCurrent();
-
-    // Loop Initialization
-    FSW_ELOG(_("Scheduling stream with run loop...\n"));
-    FSEventStreamScheduleWithRunLoop(stream,
-                                     run_loop,
-                                     kCFRunLoopDefaultMode);
+    // Creating dispatch queue
+    fsevents_queue = dispatch_queue_create("fswatch_event_queue", NULL);
+    FSEventStreamSetDispatchQueue(stream, fsevents_queue);
 
     FSW_ELOG(_("Starting event stream...\n"));
     FSEventStreamStart(stream);
@@ -137,9 +132,16 @@ namespace fsw
     run_loop_lock.unlock();
 #endif
 
-    // Loop
-    FSW_ELOG(_("Starting run loop...\n"));
-    CFRunLoopRun();
+    for(;;)
+    {
+#ifdef HAVE_CXX_MUTEX
+      run_loop_lock.lock();
+      if (should_stop) break;
+      run_loop_lock.unlock();
+#endif
+
+      std::this_thread::sleep_for(std::chrono::milliseconds((long long) (latency * 1000)));
+    }
 
     // Deinitialization part
     FSW_ELOG(_("Stopping event stream...\n"));
@@ -151,22 +153,10 @@ namespace fsw
     FSW_ELOG(_("Releasing event stream...\n"));
     FSEventStreamRelease(stream);
 
+    dispatch_release(fsevents_queue);
     stream = nullptr;
   }
 
-  /*
-   * on_stop() is designed to be invoked with a lock on the run_mutex.
-   */
-  void fsevents_monitor::on_stop()
-  {
-    if (!run_loop) throw libfsw_exception(_("run loop is null"));
-
-    FSW_ELOG(_("Stopping run loop...\n"));
-    CFRunLoopStop(run_loop);
-
-    run_loop = nullptr;
-  }
-
   static vector<fsw_event_flag> decode_flags(FSEventStreamEventFlags flag)
   {
     vector<fsw_event_flag> evt_flags;
@@ -219,6 +209,9 @@ namespace fsw
   {
     string no_defer = get_property(DARWIN_EVENTSTREAM_NO_DEFER);
 
+    if (no_defer.empty())
+      return (!isatty(fileno(stdin)));
+
     return (no_defer == "true");
   }
 
