@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2015 Enrico M. Crisostomo
+ * Copyright (c) 2014-2021 Enrico M. Crisostomo
  *
  * This program is free software; you can redistribute it and/or modify it under
  * the terms of the GNU General Public License as published by the Free Software
@@ -13,12 +13,11 @@
  * You should have received a copy of the GNU General Public License along with
  * this program.  If not, see <http://www.gnu.org/licenses/>.
  */
-#ifdef HAVE_CONFIG_H
-#  include "libfswatch_config.h"
-#endif
+#include "libfswatch/libfswatch_config.h"
 
-#include "gettext_defs.h"
+#include "libfswatch/gettext_defs.h"
 #include "inotify_monitor.hpp"
+#include <algorithm>
 #include <limits.h>
 #ifdef __sun
 #  define NAME_MAX         255    /* # chars in a file name */
@@ -35,15 +34,13 @@
 #include "libfswatch_set.hpp"
 #include "path_utils.hpp"
 
-using namespace std;
-
 namespace fsw
 {
 
   struct inotify_monitor_impl
   {
     int inotify_monitor_handle = -1;
-    vector<event> events;
+    std::vector<event> events;
     /*
      * A map of file names by descriptor is kept in sync because the name field
      * of the inotify_event structure is present only when it identifies a
@@ -51,7 +48,7 @@ namespace fsw
      * for easy retrieval.
      */
     fsw_hash_set<int> watched_descriptors;
-    fsw_hash_map<string, int> path_to_wd;
+    fsw_hash_map<std::string, int> path_to_wd;
     /*
      * Since the inotify API maintains only works with watch
      * descriptors a cache maintaining a relationship between a watch
@@ -63,17 +60,16 @@ namespace fsw
      *   between watch descriptors and pathnames.  Be aware that directory
      *   renamings may affect multiple cached pathnames.
      */
-    fsw_hash_map<int, string> wd_to_path;
+    fsw_hash_map<int, std::string> wd_to_path;
     fsw_hash_set<int> descriptors_to_remove;
     fsw_hash_set<int> watches_to_remove;
+    std::vector<std::string> paths_to_rescan;
     time_t curr_time;
   };
 
   static const unsigned int BUFFER_SIZE = (10 * ((sizeof(struct inotify_event)) + NAME_MAX + 1));
 
-  REGISTER_MONITOR_IMPL(inotify_monitor, inotify_monitor_type);
-
-  inotify_monitor::inotify_monitor(vector<string> paths_to_monitor,
+  inotify_monitor::inotify_monitor(std::vector<std::string> paths_to_monitor,
                                    FSW_EVENT_CALLBACK *callback,
                                    void *context) :
     monitor(paths_to_monitor, callback, context),
@@ -93,7 +89,7 @@ namespace fsw
     // close inotify watchers
     for (auto inotify_desc_pair : impl->watched_descriptors)
     {
-      ostringstream log;
+      std::ostringstream log;
       log << _("Removing: ") << inotify_desc_pair << "\n";
       FSW_ELOG(log.str().c_str());
 
@@ -112,7 +108,7 @@ namespace fsw
     delete impl;
   }
 
-  bool inotify_monitor::add_watch(const string& path,
+  bool inotify_monitor::add_watch(const std::string& path,
                                   const struct stat& fd_stat)
   {
     // TODO: Consider optionally adding the IN_EXCL_UNLINK flag.
@@ -130,7 +126,7 @@ namespace fsw
       impl->wd_to_path[inotify_desc] = path;
       impl->path_to_wd[path] = inotify_desc;
 
-      ostringstream log;
+      std::ostringstream log;
       log << _("Added: ") << path << "\n";
       FSW_ELOG(log.str().c_str());
     }
@@ -138,14 +134,14 @@ namespace fsw
     return (inotify_desc != -1);
   }
 
-  void inotify_monitor::scan(const string& path, const bool accept_non_dirs)
+  void inotify_monitor::scan(const std::string& path, const bool accept_non_dirs)
   {
     struct stat fd_stat;
     if (!lstat_path(path, fd_stat)) return;
 
     if (follow_symlinks && S_ISLNK(fd_stat.st_mode))
     {
-      string link_path;
+      std::string link_path;
       if (read_link_path(path, link_path))
         scan(link_path, accept_non_dirs);
 
@@ -163,18 +159,17 @@ namespace fsw
      * For the same reason, the directory_only flag is ignored and treated as if
      * it were always set to true.
      */
-    if (!accept_non_dirs && !is_dir) return;
-
-    if (!is_dir && directory_only && !accept_non_dirs) return;
-    if (!is_dir && !accept_path(path)) return;
+    if (!is_dir && !accept_non_dirs) return;
+    if (!is_dir && directory_only) return;
+    if (!accept_path(path)) return;
     if (!add_watch(path, fd_stat)) return;
     if (!recursive || !is_dir) return;
 
-    vector<string> children = get_directory_children(path);
+    std::vector<std::string> children = get_directory_children(path);
 
-    for (const string& child : children)
+    for (const std::string& child : children)
     {
-      if (child.compare(".") == 0 || child.compare("..") == 0) continue;
+      if (child == "." || child == "..") continue;
 
       /*
        * Scan children but only watch directories.
@@ -183,14 +178,14 @@ namespace fsw
     }
   }
 
-  bool inotify_monitor::is_watched(const string& path) const
+  bool inotify_monitor::is_watched(const std::string& path) const
   {
     return (impl->path_to_wd.find(path) != impl->path_to_wd.end());
   }
 
   void inotify_monitor::scan_root_paths()
   {
-    for (string& path : paths)
+    for (std::string& path : paths)
     {
       if (!is_watched(path)) scan(path);
     }
@@ -198,7 +193,7 @@ namespace fsw
 
   void inotify_monitor::preprocess_dir_event(struct inotify_event *event)
   {
-    vector<fsw_event_flag> flags;
+    std::vector<fsw_event_flag> flags;
 
     if (event->mask & IN_ISDIR) flags.push_back(fsw_event_flag::IsDir);
     if (event->mask & IN_MOVE_SELF) flags.push_back(fsw_event_flag::Updated);
@@ -208,11 +203,17 @@ namespace fsw
     {
       impl->events.push_back({impl->wd_to_path[event->wd], impl->curr_time, flags});
     }
+
+    // If a new directory has been created, it should be rescanned if the
+    if ((event->mask & IN_ISDIR) && (event->mask & IN_CREATE))
+    {
+      impl->paths_to_rescan.push_back(impl->wd_to_path[event->wd]);
+    }
   }
 
   void inotify_monitor::preprocess_node_event(struct inotify_event *event)
   {
-    vector<fsw_event_flag> flags;
+    std::vector<fsw_event_flag> flags;
 
     if (event->mask & IN_ACCESS) flags.push_back(fsw_event_flag::PlatformSpecific);
     if (event->mask & IN_ATTRIB) flags.push_back(fsw_event_flag::AttributeModified);
@@ -234,7 +235,7 @@ namespace fsw
     if (event->mask & IN_OPEN) flags.push_back(fsw_event_flag::PlatformSpecific);
 
     // Build the file name.
-    ostringstream filename_stream;
+    std::ostringstream filename_stream;
     filename_stream << impl->wd_to_path[event->wd];
 
     if (event->len > 1)
@@ -249,7 +250,7 @@ namespace fsw
     }
 
     {
-      ostringstream log;
+      std::ostringstream log;
       log << _("Generic event: ") << event->wd << "::" << filename_stream.str() << "\n";
       FSW_ELOG(log.str().c_str());
     }
@@ -260,7 +261,7 @@ namespace fsw
      */
     if (event->mask & IN_IGNORED)
     {
-      ostringstream log;
+      std::ostringstream log;
       log << "IN_IGNORED: " << event->wd << "::" << filename_stream.str() << "\n";
       FSW_ELOG(log.str().c_str());
 
@@ -281,7 +282,7 @@ namespace fsw
      */
     if (event->mask & IN_MOVE_SELF)
     {
-      ostringstream log;
+      std::ostringstream log;
       log << "IN_MOVE_SELF: " << event->wd << "::" << filename_stream.str() << "\n";
       FSW_ELOG(log.str().c_str());
 
@@ -290,7 +291,7 @@ namespace fsw
     }
 
     /*
-     * An file could be moved to a path which is being observed.  The clobbered
+     * A file could be moved to a path which is being observed.  The clobbered
      * file is handled by the corresponding IN_DELETE_SELF event.
      */
 
@@ -300,7 +301,7 @@ namespace fsw
      */
     if (event->mask & IN_DELETE_SELF)
     {
-      ostringstream log;
+      std::ostringstream log;
       log << "IN_DELETE_SELF: " << event->wd << "::" << filename_stream.str() << "\n";
       FSW_ELOG(log.str().c_str());
 
@@ -341,7 +342,7 @@ namespace fsw
       }
       else
       {
-        ostringstream log;
+        std::ostringstream log;
         log << _("Removed: ") << *wtd << "\n";
         FSW_ELOG(log.str().c_str());
       }
@@ -354,13 +355,24 @@ namespace fsw
 
     while (fd != impl->descriptors_to_remove.end())
     {
-      const string& curr_path = impl->wd_to_path[*fd];
+      const std::string& curr_path = impl->wd_to_path[*fd];
       impl->path_to_wd.erase(curr_path);
       impl->wd_to_path.erase(*fd);
       impl->watched_descriptors.erase(*fd);
 
       impl->descriptors_to_remove.erase(fd++);
     }
+
+    // Process paths to be rescanned
+    std::for_each(impl->paths_to_rescan.begin(),
+		  impl->paths_to_rescan.end(),
+		  [this] (const std::string& p)
+		  {
+		    this->scan(p);
+		  }
+		  );
+
+    impl->paths_to_rescan.clear();
   }
 
   void inotify_monitor::run()
@@ -372,7 +384,7 @@ namespace fsw
     for(;;)
     {
 #ifdef HAVE_CXX_MUTEX
-      unique_lock<mutex> run_guard(run_mutex);
+      std::unique_lock<std::mutex> run_guard(run_mutex);
       if (should_stop) break;
       run_guard.unlock();
 #endif
@@ -407,7 +419,8 @@ namespace fsw
 
       if (rv == -1)
       {
-        throw libfsw_exception(_("::select() on inotify descriptor encountered an error."));
+	fsw_log_perror("select");
+	continue;
       }
 
       // In case of read timeout just repeat the loop.
@@ -418,7 +431,7 @@ namespace fsw
                                 BUFFER_SIZE);
 
       {
-        ostringstream log;
+        std::ostringstream log;
         log << _("Number of records: ") << record_num << "\n";
         FSW_ELOG(log.str().c_str());
       }
